//+------------------------------------------------------------------+
//|                                        WeeklyPivotsSimulator.mq4 |
//|                      Copyright  2012, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright  2012, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net"
#include <WinUser32.mqh>
#include <stdlib.mqh>

#property indicator_chart_window
#property indicator_buffers 1

extern string wmtext1                = "pivot timeframe";
extern string wmtext2                = "weeky=1, monthly=2";
extern int    pivot_timeframe        = 1;
extern string mltext1                = "first instance supports main pivot levels";
extern string mltext2                = "to add mid pivot levels, load this";
extern string mltext3                = "indicator a second time and set below true";
extern bool   addMidLevels           = false;
extern string bartext                = "set to number of bars to use";
extern int    BarsToProcess          = 10;
extern string linetext               = "below line/text style is configured";
extern string text_font              = "Arial";
extern int    text_size              = 7;
extern color  text_color             = Red;
extern string tstext1                = "text_shift shifts the line text by N spaces";
extern int    text_shift             = 40;
extern int    line_thickness_main    = 2;
extern int    line_thickness_mid     = 1;
extern color  line_color_main        = Black;           // Black
extern color  line_color_support1    = DodgerBlue;      // DodgerBlue
extern color  line_color_resistance1 = OrangeRed;       // OrangeRed
extern color  line_color_support2    = RoyalBlue;       // RoyalBlue
extern color  line_color_resistance2 = Crimson;         // Crimson
extern color  line_color_support3    = Blue;            // Blue
extern color  line_color_resistance3 = Maroon;          // Maroon
extern string tgtext1                = "false=off, true=on";
extern bool   Tick_Generator         = false;

//---- buffers
double PBuffer[];

//---- global variables
bool     Tripwire1, Tripwire2, Tripwire3, Tripwire4, Tripwire5;
color    lc[7];
datetime current_time, Time1, Time2, Time3, Time4, Time5, OT1, OT2;
datetime trn_beg, trn_end, piv_beg, piv_end, ext_beg, ext_end;
datetime drw_trn_beg, drw_trn_end, drw_piv_beg, drw_piv_end, drw_ext_beg, drw_ext_end;
double   multiplier, NP[7], OP[7];
double   last_high, last_low, last_close, curr_high, curr_low, curr_close;
double   cP, cS1, cR1, cS2, cR2, cS3, cR3;
double   cMS1, cMS2, cMS3, cMR1, cMR2, cMR3;
double   lP, lS1, lR1, lS2, lR2, lS3, lR3;
double   lMS1, lMS2, lMS3, lMR1, lMR2, lMR3;
int      incr;
int      i, k, err;
int      line_thickness;
int      pivot_bar;
int      pivot_timeframe_const;
int      lt[2];
string   pregap = "";
string   timeframe_prefix;
string   timeframe_prefix_text;
string   stg1, stg2, stg3, stg4;
string   se_name[7];
string   sp_name[7];
string   st_name[7];
string   stx_name[7];

// MT4Ticker
datetime Timer;
datetime Timer_Second;
// MT4Ticker
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   //Accommodate different quote sizes
   if(StringFind(Symbol(),"JPY",0)<0)  multiplier = 10000;
   if(StringFind(Symbol(),"JPY",0)>=0) multiplier = 100;

   if(Tick_Generator==true) Timer = D'2012.05.12 00:00';
   Timer_Second = 1;

   Tripwire1=true;
   Tripwire2=true;
   Tripwire3=true;
   Tripwire4=true;
   Tripwire5=true;

   se_name[0]="Sim Pivot Extension"; se_name[1]="Sim Sup1 Extension"; se_name[2]="Sim Res1 Extension";
   se_name[3]="Sim Sup2 Extension"; se_name[4]="Sim Res2 Extension"; se_name[5]="Sim Sup3 Extension";
   se_name[6]="Sim Res3 Extension";

   sp_name[0]="Sim Pivot"; sp_name[1]="Sim Sup1"; sp_name[2]="Sim Res1"; sp_name[3]="Sim Sup2";
   sp_name[4]="Sim Res2"; sp_name[5]="Sim Sup3"; sp_name[6]="Sim Res3";

   st_name[0]="Sim Pivot Transition"; st_name[1]="Sim Sup1 Transition"; st_name[2]="Sim Res1 Transition";
   st_name[3]="Sim Sup2 Transition"; st_name[4]="Sim Res2 Transition"; st_name[5]="Sim Sup3 Transition";
   st_name[6]="Sim Res3 Transition";

   stx_name[0]="Sim Pivot"; stx_name[1]="Sim Sup1"; stx_name[2]="Sim Res1"; stx_name[3]="Sim Sup2";
   stx_name[4]="Sim Res2"; stx_name[5]="Sim Sup3"; stx_name[6]="Sim Res3";
   
   lt[0]=line_thickness_main; lt[1]=line_thickness_mid;
   
   lc[0]=line_color_main; lc[1]=line_color_support1; lc[2]=line_color_resistance1;
   lc[3]=line_color_support2; lc[4]=line_color_resistance2; lc[5]=line_color_support2;
   lc[6]=line_color_resistance3;

   if(addMidLevels == true) line_thickness = line_thickness_mid;
   else line_thickness = line_thickness_main;

   SetIndexBuffer(0,PBuffer);
     
   SetIndexStyle(0,DRAW_LINE,STYLE_SOLID,line_thickness, line_color_main);
  
   if(pivot_timeframe == 1) pivot_timeframe_const = PERIOD_W1;
   if(pivot_timeframe == 2) pivot_timeframe_const = PERIOD_MN1;
   
   if(pivot_timeframe == 1) timeframe_prefix = "Weekly ";
   if(pivot_timeframe == 2) timeframe_prefix = "Monthly ";
   
   pregap = "";
   for(k=0; k<text_shift;k++) pregap = pregap + " ";
   if(addMidLevels == true) timeframe_prefix = timeframe_prefix + "Mid ";
   timeframe_prefix_text = pregap + timeframe_prefix;
      
   for(k=0;k<=6;k++)
   {
     stg1=StringConcatenate(timeframe_prefix, sp_name[k]);
     SetIndexLabel(k,stg1);
   }
   //+--------------------------------------------+
   //           Simulation Graphics               |
   //+--------------------------------------------+ 
   stg1 = StringConcatenate(timeframe_prefix,se_name[0]);
   err = GetLastError();
   if(err!=4200)   // error 4200 means objects already exists               
   {
     for(k=0;k<=6;k++)
     {
       // simulated pivot transitions (used during periods of Closed Market)
       stg1=StringConcatenate(timeframe_prefix, st_name[k]);
       ObjectCreate(stg1, OBJ_TREND, 0, 0, 0, 0, 0);   //  sim pivot transitions
       ObjectSet(stg1, OBJPROP_COLOR, lc[k]);
       ObjectSet(stg1, OBJPROP_STYLE, STYLE_SOLID);
       ObjectSet(stg1, OBJPROP_WIDTH, line_thickness);
       ObjectSet(stg1, OBJPROP_RAY, FALSE);
      
       // simulated pivot lines (used during periods of Closed Market)
       stg1 = StringConcatenate(timeframe_prefix, sp_name[k]);
       ObjectCreate(stg1, OBJ_TREND, 0, 0, 0, 0, 0);   // sim pivot lines
       ObjectSet(stg1, OBJPROP_COLOR, lc[k]);
       ObjectSet(stg1, OBJPROP_STYLE, STYLE_SOLID);
       ObjectSet(stg1, OBJPROP_WIDTH, line_thickness);
       ObjectSet(stg1, OBJPROP_RAY, FALSE);

     // simulated pivot extensions
     stg1 = StringConcatenate(timeframe_prefix, se_name[k]);
     ObjectCreate(stg1, OBJ_TREND, 0, 0, 0, 0, 0);   // Sim Extensions
     ObjectSet(stg1, OBJPROP_COLOR, lc[k]);
     ObjectSet(stg1, OBJPROP_STYLE, STYLE_DOT);

       // simulated text
       stg1 = StringConcatenate(timeframe_prefix, stx_name[k]);
       stg2 = StringConcatenate(timeframe_prefix_text, " ", stx_name[k]);
       ObjectCreate(stg1, OBJ_TEXT, 0, 0, 0);   // sim text
       ObjectSetText(stg1, stg2, text_size, text_font, text_color);
     }
   }

   WindowRedraw();   
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
   int k;
   for(k=ObjectsTotal()-1;k>=0;k--)  // delete all objects with "Sim "
   {                                // starting in position 7
     if(StringFind(ObjectName(k),"Sim ",0)==7)
     {
       ObjectDelete(ObjectName(k));
     }
   }
   WindowRedraw();
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
  // MT4Ticker
   if(Tick_Generator==false) Timer=TimeCurrent();
   Timer = Timer + Timer_Second;
   //   Alert("Timer:", Timer, "  ", TimeToStr(Timer, TIME_DATE|TIME_SECONDS));
   // MT4Ticker

   int i,counted_bars=IndicatorCounted();
   //---- check for possible errors
   if(counted_bars<0) return(-1);
   //---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;  
   int Limit=Bars-counted_bars;

   if(pivot_timeframe == 1) if(Period()>PERIOD_D1) return(-1);
   if(pivot_timeframe == 2) if(Period()>PERIOD_W1) return(-1);

   if(Limit>BarsToProcess) Limit=BarsToProcess;
   for(i=Limit-1; i>0; i--)
   {
   }   

   //+------------------ Times being adjusted
   if(Timer>=Time1 && Tripwire1==true)
   {
     Tripwire1=false;
     Print("Times are being adjusted", "  ", TimeToStr(Timer, TIME_DATE|TIME_SECONDS));
     Time2 = Timer +   2;   //  0 seconds    Display Simulated    
     Time3 = Timer +  16;   // 15 seconds    Remove Simulated
     Time1 = Timer +  30;   // 30 seconds    Adjust Times

     Tripwire2=true;
     WindowRedraw();
   }

   // On Fridays between 15:59 and 16:00 Hours (Market Closes at 16:00 hours),
   // insert simulated graphics to display next weeks pivots.

   //+------------------ Move (Draw on Screen) Simulated Graphics       
   if(Timer>=Time2 && Tripwire2==true)
   {
     Tripwire2=false;

     // get times from original pivot ext
     OT1 = ObjectGet(StringConcatenate(timeframe_prefix, "Pivot ext"), OBJPROP_TIME1);
     OT2 = ObjectGet(StringConcatenate(timeframe_prefix, "Pivot ext"), OBJPROP_TIME2);
   
     // looking for the current time   
     current_time = Time[i];
     // and seach the corresponding bar in the pivot timeframe
     pivot_bar = iBarShift( NULL, pivot_timeframe_const, current_time);

     // read the values of the current week/month
     last_low = iLow(NULL, pivot_timeframe_const, pivot_bar+1);
     last_high = iHigh(NULL, pivot_timeframe_const, pivot_bar+1);
     last_close = iClose(NULL, pivot_timeframe_const, pivot_bar+1);

     curr_low = iLow(NULL, pivot_timeframe_const, pivot_bar+0);
     curr_high = iHigh(NULL, pivot_timeframe_const, pivot_bar+0);
     curr_close = iClose(NULL, pivot_timeframe_const, pivot_bar+0);

     // calculate main pivot and S/R lines		    
     lP =(last_high+last_low+last_close)/3;  cP =(curr_high+curr_low+curr_close)/3;
     lR1=(2*lP)-last_low;                    cR1=(2*cP)-curr_low;
     lS1=(2*lP)-last_high;                   cS1=(2*cP)-curr_high;
     lR2=lP+(last_high-last_low);            cR2=cP+(curr_high-curr_low);
     lS2=lP-(last_high-last_low);            cS2=cP-(curr_high-curr_low);
     lR3=(2*lP)+(last_high-(2*last_low));    cR3=(2*cP)+(curr_high-(2*curr_low));
     lS3=(2*lP)-((2*last_high)-last_low);    cS3=(2*cP)-((2*curr_high)-curr_low);

     // calculate mid S/R lines		    
     lMS3 = lS3 + ((lS2-lS3) / 2);           cMS3 = cS3 + ((cS2-cS3) / 2);
     lMS2 = lS2 + ((lS1-lS2) / 2);           cMS2 = cS2 + ((cS1-cS2) / 2);
     lMS1 = lS1 + ((lP-lS1)  / 2);           cMS1 = cS1 + ((cP-cS1)  / 2);
     lMR1 = lP  + ((lR1-lP)  / 2);           cMR1 = cP  + ((cR1-cP)  / 2);
     lMR2 = lR1 + ((lR2-lR1) / 2);           cMR2 = cR1 + ((cR2-cR1) / 2);
     lMR3 = lR2 + ((lR3-lR2) / 2);           cMR3 = cR2 + ((cR3-cR2) / 2);
       
     // copy either main or mid values to the temp display variables
     if( addMidLevels == false)
     {
       OP[0]=lP;                            NP[0]=cP;            
       OP[1]=lS1;                           NP[1]=cS1;
       OP[2]=lR1;                           NP[2]=cR1;
       OP[3]=lS2;                           NP[3]=cS2;
       OP[4]=lR2;                           NP[4]=cR2;
       OP[5]=lS3;                           NP[5]=cS3;
       OP[6]=lR3;                           NP[6]=cR3;
     }
     else
     {
       OP[0]=0;                             NP[0]=0;
       OP[1]=lMS1;                          NP[1]=cMS1;
       OP[2]=lMR1;                          NP[2]=cMR1;
       OP[3]=lMS2;                          NP[3]=cMS2;
       OP[4]=lMR2;                          NP[4]=cMR2;
       OP[5]=lMS3;                          NP[5]=cMS3;
       OP[6]=lMR3;                          NP[6]=cMR3;
     }

     Print("Simulated Graphics being drawn", "  ", TimeToStr(Timer, TIME_DATE|TIME_SECONDS));

     pregap = "";
     for(k=0; k<text_shift;k++) pregap = pregap + " ";
     if(addMidLevels == true) timeframe_prefix = timeframe_prefix + "Mid ";
     timeframe_prefix_text = pregap + timeframe_prefix;
     
     incr=Period();
     trn_beg = Time3;                // start of original extension, start of sim trn
     trn_end = trn_beg + incr*60;    // end of sim trn
     piv_beg = trn_end;              // start of sim pivot
     piv_end = piv_beg + 6*incr*60;  // end of sim pivot
     ext_beg = piv_end;              // start of sim ext
     ext_end = ext_beg + 6*incr*60;  // end of sim ext 

     for(k=0;k<=6;k++)
     {
       // draw the simulated transition, pivot, extension & text
       stg1 = StringConcatenate(timeframe_prefix, st_name[k]);
       ObjectMove(stg1, 0, trn_beg, OP[k]);       // sim transition
       ObjectMove(stg1, 1, trn_end, NP[k]);
       drw_trn_beg = trn_beg; drw_trn_end = trn_end;

       stg1 = StringConcatenate(timeframe_prefix, sp_name[k]);
       ObjectMove(stg1, 0, piv_beg, NP[k]);    // sim pivot
       ObjectMove(stg1, 1, piv_end, NP[k]);
       drw_piv_beg = piv_beg; drw_piv_end = piv_end;

       stg1 = StringConcatenate(timeframe_prefix, se_name[k]);
       ObjectMove(stg1, 0, ext_beg, NP[k]);     // sim extension
       ObjectMove(stg1, 1, ext_end, NP[k]);
       drw_ext_beg = ext_beg; drw_ext_end = ext_end;
     }

     Tripwire3=true;
     WindowRedraw();
   }

   // On Sunday just after 22:00 Hours (Market Opens at approximately 22:00),
   // remove simulated graphics

   //+------------------ Move (Clear from Screen} Simulated Graphics       
   if(Timer>=Time3 && Tripwire3==true)
   {
     Tripwire3=false;
     Print("Simulated Graphics being moved off screen", "  ", TimeToStr(Timer, TIME_DATE|TIME_SECONDS));
     for(k=0;k<=6;k++)
     {
       stg1 = StringConcatenate(timeframe_prefix, st_name[k]);
       ObjectMove(stg1, 0, drw_trn_beg, 0);
       ObjectMove(stg1, 1, drw_trn_end, 0);
       
       stg1 = StringConcatenate(timeframe_prefix, sp_name[k]);
       ObjectMove(stg1, 0, drw_piv_beg, 0);
       ObjectMove(stg1, 1, drw_piv_end, 0);
       
       stg1 = StringConcatenate(timeframe_prefix, se_name[k]);
       ObjectMove(stg1, 0, drw_ext_beg, 0);
       ObjectMove(stg1, 1, drw_ext_end, 0);
     }

     Tripwire1=true;
     WindowRedraw();
   }
   //----
   return(0);
}

//+------------------------------------------------------------------+

