//+------------------------------------------------------------------+
//|                                           iExposure, Tickets.mq4 |
//|                        Copyright 2007, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//|    UPGRADED BY BALUDA 5/25/2012                                  |
//+------------------------------------------------------------------+
#property copyright "Copyright 2007, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net"

#property indicator_separate_window
#property indicator_buffers 1
#property indicator_minimum 0.0
#property indicator_maximum 0.1

#define EPSILON      0.00000001

#define TICKETS_MAX  1024
#define SYMBOL       0
#define TICKET       1
#define TIME         2
#define TYPE         3
#define LOTS         4
#define PRICE        5
#define PROFIT       6
#define PIPS         7
#define STOPLOSS     8
#define TAKEPROFIT   9
#define MESSAGE      10

#define COLUMNS      11

extern string  ste            = "----Stealth----";
extern bool    useGVStealth   = false;
extern string  stealthPrefix  = "Stealth_";
extern bool    useHiddenPips  = false;
extern int     hiddenPips     = 0;

extern string  gen            = "----General----";
extern bool    showRealPips   = true;
extern string  nonPropFont    = "Lucida Console";

extern string  col            = "----Colo(u)rs----";
extern color   ExtHeaderColor = DarkSlateBlue;
extern color   ExtBodyColor   = CadetBlue;
extern color   ExtWinBGColor  = SeaGreen; // LightGreen;
extern color   ExtLossBGColor = FireBrick; // LightPink;
extern color   ExtWinColor    = SeaGreen;
extern color   ExtLossColor   = FireBrick;

string ExtName = "Exposure, Tickets";
string ExtTickets[TICKETS_MAX][COLUMNS];
double ExtTicketsValues[TICKETS_MAX][COLUMNS];
int    ExtLines = -1;
string ExtCols[COLUMNS] = {"Symbol", "Ticket", "Time", "Type", " Lots", " Price", "Profit", "Pips", " Stoploss", 
                           "Takeprofit", " " };
int    ExtShifts[COLUMNS] = { 10, 70, 150, 280, 320, 365, 440, 510, 585, 670, 760 };
int    ExtVertShift = 14;
int    ExtVertOffset = 8;
double ExtMapBuffer[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
void init()
{
   IndicatorShortName ( ExtName );
   SetIndexBuffer ( 0, ExtMapBuffer );
   SetIndexStyle ( 0, DRAW_NONE );
   IndicatorDigits ( 0 );
   SetIndexEmptyValue ( 0, 0.0 );
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void deinit()
{
   int windex = WindowFind ( ExtName );
   if ( windex > 0 ) ObjectsDeleteAll ( windex );
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
void start()
{
   string name;
   int    i, col, line, windex = WindowFind ( ExtName );
   //----
   if ( windex < 0 ) return;

   //---- header line
   if ( ExtLines < 0 )
   {
      for ( col = 0; col < COLUMNS; col++ )
      {
         name = "Header_" + col;
         if ( ObjectCreate ( name, OBJ_LABEL, windex, 0, 0 ) )
         {
            ObjectSet ( name, OBJPROP_XDISTANCE, ExtShifts[col] );
            ObjectSet ( name, OBJPROP_YDISTANCE, ExtVertShift + ExtVertOffset - 1 );
            ObjectSetText ( name, ExtCols[col], 9, nonPropFont, ExtHeaderColor );
         }
      }
      ExtLines = 0;
   }
   //----

   double footer_profit = 0.0;
   double footer_pips = 0.0;
   int total = Analyze();
   if ( total > 0 )
   {
      for ( line = 1; line <= total; line++ )
      {
         //---- add line
         name = "Line_" + line + "_0";
         if ( ObjectFind(name) < 0 )
         {
            int y_dist = ExtVertShift * ( line + 1 ) + ExtVertOffset;
            for ( col = 0; col < COLUMNS; col++ )
            {
               name = "Line_" + line + "_" + col;
               if ( ObjectCreate ( name, OBJ_LABEL, windex, 0, 0 ) )
               {
                  ObjectSet ( name, OBJPROP_XDISTANCE, ExtShifts[col] );
                  ObjectSet ( name, OBJPROP_YDISTANCE, y_dist );
               }
            }
         }

         //---- set line
         int    digits = MarketInfo ( ExtTickets[line - 1][SYMBOL], MODE_DIGITS );
         int    trailing_spaces = ( digits < 4 ) * 2;

         color line_color = ExtLossColor;
         color background_color = ExtLossBGColor;
         if ( ExtTicketsValues[line - 1][PROFIT] > 0 )
         {
            background_color = ExtWinBGColor;
         }
         if ( ( ExtTicketsValues[line - 1][TYPE] == OP_BUY && ExtTicketsValues[line - 1][STOPLOSS] > ExtTicketsValues[line - 1][PRICE] )
              ||
              ( ExtTicketsValues[line - 1][TYPE] == OP_SELL && ExtTicketsValues[line - 1][STOPLOSS] < ExtTicketsValues[line - 1][PRICE] )
            )
         {
            line_color = ExtWinColor;
         }
         footer_profit += ExtTicketsValues[line - 1][PROFIT];
         footer_pips += ExtTicketsValues[line - 1][PIPS];
         
         name = "Line_" + line + "_0";
         ObjectSetText ( name, ExtTickets[line - 1][SYMBOL], 9, nonPropFont, ExtBodyColor ); // line_color
         
         name = "Line_" + line + "_1";
         ObjectSetText ( name, RightAlign ( ExtTickets[line - 1][TICKET], 3 ), 9, nonPropFont, ExtBodyColor );
         
         name = "Line_" + line + "_2";
         ObjectSetText ( name, RightAlign ( ExtTickets[line - 1][TIME], 3 ), 9, nonPropFont, ExtBodyColor );
         
         name = "Line_" + line + "_3";
         ObjectSetText ( name, RightAlign ( ExtTickets[line - 1][TYPE], 3), 9, nonPropFont, ExtBodyColor );
         
         name = "Line_" + line + "_4";
         ObjectSetText ( name, RightAlign ( ExtTickets[line - 1][LOTS], 5 ), 9, nonPropFont, ExtBodyColor );
         
         name = "Line_" + line + "_5";
         ObjectSetText ( name, RightAlign ( ExtTickets[line - 1][PRICE], 9, trailing_spaces ), 9, nonPropFont, ExtBodyColor );
         
         name = "Line_" + line + "_6";
         ObjectSetText ( name, RightAlign ( ExtTickets[line - 1][PROFIT], 8 ), 9, nonPropFont, White );
         name = "Line_" + line + "_6_background";
         DrawCell ( windex, name, ExtShifts[6], ExtVertShift * ( line + 1 ) + ExtVertOffset - 7, 60, 13, background_color );
         
         name = "Line_" + line + "_7";
         ObjectSetText ( name, RightAlign ( ExtTickets[line - 1][PIPS], 8 ), 9, nonPropFont, White );
         name = "Line_" + line + "_7_background";
         DrawCell ( windex, name, ExtShifts[7], ExtVertShift * ( line + 1 ) + ExtVertOffset - 7, 60, 13, background_color );

         name = "Line_" + line + "_8";
         ObjectSetText ( name, RightAlign ( ExtTickets[line - 1][STOPLOSS], 9, trailing_spaces ), 9, nonPropFont, line_color );

         //--- Show bullet for imminent stoploss
         name = "Line_" + line + "_8_background";
         double current_price = MarketInfo(ExtTickets[line - 1][SYMBOL], MODE_BID);
         if (ExtTicketsValues[line - 1][TYPE] == OP_SELL) current_price = MarketInfo(ExtTickets[line - 1][SYMBOL], MODE_ASK);
         if ( MathAbs((ExtTicketsValues[line - 1][STOPLOSS] - current_price) * ((digits > 3) * 99 + 1)) < 0.1 )
         {
            DrawBullet( windex, name, ExtShifts[8] + 65, ExtVertShift * ( line + 1 ) + ExtVertOffset - 2, ExtLossBGColor );
         }
         else
         {
            if (ObjectFind(name) > -1) ObjectDelete(name);
         }
         //---

         name = "Line_" + line + "_9";
         ObjectSetText ( name, RightAlign ( ExtTickets[line - 1][TAKEPROFIT], 9, trailing_spaces ), 9, nonPropFont, ExtBodyColor );

         //--- Show bullet for imminent takeprofit
         name = "Line_" + line + "_9_background";
         current_price = MarketInfo(ExtTickets[line - 1][SYMBOL], MODE_BID);
         if (ExtTicketsValues[line - 1][TYPE] == OP_SELL) current_price = MarketInfo(ExtTickets[line - 1][SYMBOL], MODE_ASK);
         if ( MathAbs((ExtTicketsValues[line - 1][TAKEPROFIT] - current_price) * ((digits > 3) * 99 + 1)) < 0.1 )
         {
            DrawBullet( windex, name, ExtShifts[9] + 65, ExtVertShift * ( line + 1 ) + ExtVertOffset - 2, ExtWinBGColor );
         }
         else
         {
            if (ObjectFind(name) > -1) ObjectDelete(name);
         }
         //---

         name = "Line_" + line + "_10";
         ObjectSetText ( name, RightAlign ( ExtTickets[line - 1][MESSAGE], 6 ), 9, nonPropFont, ExtBodyColor );
      }
   }
   
   if ( line == 0 ) line++;

   //---- footer line
   name = "Footer_0";
   int FooterVertOffset = 3;
   if ( ObjectFind(name) < 0 )
   {
      for ( col = 0; col < COLUMNS; col++ )
      {
         name = "Footer_" + col;
         if ( ObjectCreate ( name, OBJ_LABEL, windex, 0, 0 ) )
         {
            ObjectSet ( name, OBJPROP_XDISTANCE, ExtShifts[col] );
            ObjectSet ( name, OBJPROP_YDISTANCE, ExtVertShift * ( line + 1 ) + ExtVertOffset + FooterVertOffset );
            ObjectSetText ( name, "" );
         }
      }
   }
   background_color = ExtLossBGColor;
   if ( footer_profit > 0 ) background_color = ExtWinBGColor;
   
   name = "Footer_0";
   ObjectSet ( name, OBJPROP_YDISTANCE, ExtVertShift * ( line + 1 ) + ExtVertOffset + FooterVertOffset );
   ObjectSetText ( name, "" );

   name = "Footer_1";
   ObjectSet ( name, OBJPROP_YDISTANCE, ExtVertShift * ( line + 1 ) + ExtVertOffset + FooterVertOffset );
   ObjectSetText ( name, "" );

   name = "Footer_2";
   ObjectSet ( name, OBJPROP_YDISTANCE, ExtVertShift * ( line + 1 ) + ExtVertOffset + FooterVertOffset );
   ObjectSetText ( name, "" );

   name = "Footer_3";
   ObjectSet ( name, OBJPROP_YDISTANCE, ExtVertShift * ( line + 1 ) + ExtVertOffset + FooterVertOffset );
   ObjectSetText ( name, "" );

   name = "Footer_4";
   ObjectSet ( name, OBJPROP_YDISTANCE, ExtVertShift * ( line + 1 ) + ExtVertOffset + FooterVertOffset );
   ObjectSetText ( name, "" );

   name = "Footer_5";
   ObjectSet ( name, OBJPROP_YDISTANCE, ExtVertShift * ( line + 1 ) + ExtVertOffset + FooterVertOffset );
   ObjectSetText ( name, "   Total", 9, nonPropFont, ExtHeaderColor );
   
   name = "Footer_6";
   ObjectSet ( name, OBJPROP_YDISTANCE, ExtVertShift * ( line + 1 ) + ExtVertOffset + FooterVertOffset );
   ObjectSetText ( name, RightAlign ( DoubleToStr ( footer_profit, 2 ), 8 ), 9, nonPropFont, White );
   name = "Footer_6_background";
   DrawCell(windex, name, ExtShifts[6], ExtVertShift*(line+1)+ExtVertOffset+FooterVertOffset-7, 60, 13, background_color);
   
   name = "Footer_7";
   ObjectSet ( name, OBJPROP_YDISTANCE, ExtVertShift * ( line + 1 ) + ExtVertOffset + FooterVertOffset );
   ObjectSetText ( name, RightAlign ( DoubleToStr ( footer_pips, 1 ), 8 ), 9, nonPropFont, White );
   name = "Footer_7_background";
   DrawCell(windex, name, ExtShifts[7], ExtVertShift*(line+1)+ExtVertOffset+FooterVertOffset-7, 60, 13, background_color);

   name = "Footer_8";
   ObjectSet ( name, OBJPROP_YDISTANCE, ExtVertShift * ( line + 1 ) + ExtVertOffset + FooterVertOffset );
   ObjectSetText ( name, "" );

   name = "Footer_9";
   ObjectSet ( name, OBJPROP_YDISTANCE, ExtVertShift * ( line + 1 ) + ExtVertOffset + FooterVertOffset );
   ObjectSetText ( name, "" );

   name = "Footer_10";
   ObjectSet ( name, OBJPROP_YDISTANCE, ExtVertShift * ( line + 1 ) + ExtVertOffset + FooterVertOffset );
   ObjectSetText ( name, "" );
   // ObjectSetText ( name, "line = " + line, 9, "Lucida Console", ExtHeaderColor );
   //----

   //---- remove lines
   while ( ObjectFind("Line_" + line + "_0") > -1 )
   {
      ObjectDelete ( "Line_" + line + "_0" );
      ObjectDelete ( "Line_" + line + "_1" );
      ObjectDelete ( "Line_" + line + "_2" );
      ObjectDelete ( "Line_" + line + "_3" );
      ObjectDelete ( "Line_" + line + "_4" );
      ObjectDelete ( "Line_" + line + "_5" );
      ObjectDelete ( "Line_" + line + "_6" );
      DeleteCell("Line_" + line + "_6_background");
      ObjectDelete ( "Line_" + line + "_7" );
      DeleteCell("Line_" + line + "_7_background");
      ObjectDelete ( "Line_" + line + "_8" );
      name = "Line_" + line + "_8_background";
      if (ObjectFind(name) > -1) ObjectDelete(name);
      ObjectDelete ( "Line_" + line + "_9" );
      name = "Line_" + line + "_9_background";
      if (ObjectFind(name) > -1) ObjectDelete(name);
      ObjectDelete ( "Line_" + line + "_10" );
      ObjectDelete ( "Line_" + line + "_11" );

      line++;
   }
//---- to avoid minimum==maximum
   ExtMapBuffer[Bars - 1] = -1;
//----
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int Analyze()
{
   int    index = 0;
   int    i, type, total = OrdersTotal();
//----
   for ( i = 0; i < total; i++ )
   {
      if ( !OrderSelect ( i, SELECT_BY_POS ) ) continue;
      type = OrderType();
      if ( type != OP_BUY && type != OP_SELL ) continue;
      
      int digits = MarketInfo ( OrderSymbol(), MODE_DIGITS );
      double multiplier = ( ( digits % 2 ) * 9 + 1 ) * MarketInfo ( OrderSymbol(), MODE_POINT );
      double profit = OrderProfit() + OrderSwap() + OrderCommission();
      double stoploss = OrderStopLoss();
      double takeprofit = OrderTakeProfit();
      if ( useGVStealth )
      {
         stoploss = GlobalVariableGet(stealthPrefix + OrderTicket() + "_SL");
         takeprofit = GlobalVariableGet(stealthPrefix + OrderTicket() + "_TP");
      }
      else if ( useHiddenPips )
      {
         if ( type == OP_BUY )
         {
            if ( stoploss > EPSILON ) stoploss += hiddenPips * multiplier;
            if ( takeprofit > EPSILON ) takeprofit -= hiddenPips * multiplier;
         }
         else if ( type == OP_SELL )
         {
            if ( stoploss > EPSILON ) stoploss -= hiddenPips * multiplier;
            if ( takeprofit > EPSILON ) takeprofit += hiddenPips * multiplier;
         }
      }

      ExtTickets[index][SYMBOL]           = OrderSymbol();
      ExtTickets[index][TICKET]           = OrderTicket();
      ExtTickets[index][TIME]             = TimeToStr(OrderOpenTime(), TIME_DATE|TIME_MINUTES);
      ExtTicketsValues[index][TYPE]       = OrderType();
      if ( type == OP_BUY )
         ExtTickets[index][TYPE]          = "Buy";
      else
         ExtTickets[index][TYPE]          = "Sell";
      ExtTickets[index][LOTS]             = DoubleToStr(OrderLots(), 2);
      ExtTicketsValues[index][PRICE]      = OrderOpenPrice();
      ExtTickets[index][PRICE]            = DoubleToStr(OrderOpenPrice(), digits);
      ExtTicketsValues[index][PROFIT]     = profit;
      ExtTickets[index][PROFIT]           = DoubleToStr(profit, 2);
      ExtTicketsValues[index][PIPS]       = profit / OrderLots() / MarketInfo( OrderSymbol(), MODE_TICKVALUE );
      if (showRealPips)
         ExtTicketsValues[index][PIPS]   /= MathMod ( MarketInfo ( OrderSymbol(), MODE_DIGITS ), 2 ) * 9 + 1;
      ExtTickets[index][PIPS]             = DoubleToStr(ExtTicketsValues[index][PIPS], 1);
      ExtTicketsValues[index][STOPLOSS]   = stoploss;
      ExtTickets[index][STOPLOSS]         = "";
      if (stoploss > EPSILON)
         ExtTickets[index][STOPLOSS]      = DoubleToStr(stoploss, digits);
      ExtTicketsValues[index][TAKEPROFIT] = takeprofit;
      ExtTickets[index][TAKEPROFIT]       = "";
      if (takeprofit > EPSILON)
         ExtTickets[index][TAKEPROFIT]    = DoubleToStr(takeprofit, digits);
      ExtTickets[index][MESSAGE]          = "";

      //----
      index += 1;
   }
//----
   return ( index );
}

//+------------------------------------------------------------------+
//| Right Align Text                                                 |
//+------------------------------------------------------------------+
string RightAlign ( string text, int length = 10, int trailing_spaces = 0 )
{
   string text_aligned = text;
   for ( int i = 0; i < length - StringLen ( text ) - trailing_spaces; i++ )
   {
      text_aligned = " " + text_aligned;
   }
   return ( text_aligned );
}

//=========================================================
// DrawCell.
//
// Author:        Alexandre A. B. Borela
// Description:   Draws a cell using the minimum character
//                as it can, so it's going to take less time
//                to draw the cell.
void DrawCell ( int nWindow, string nCellName, double nX, double nY, double nWidth, double nHeight, color nColor )
{
   double   iHeight, iWidth, iXSpace;
   int      iSquares, i;

   if ( nWidth > nHeight )
   {
      iSquares = MathCeil ( nWidth / nHeight ); // Number of squares used.
      iHeight  = MathRound ( ( nHeight * 100 ) / 77 ); // Real height size.
      iWidth   = MathRound ( ( nWidth * 100 ) / 77 ); // Real width size.
      iXSpace  = iWidth / iSquares - ( ( iHeight / ( 9 - ( nHeight / 100 ) ) ) * 2 );

      for ( i = 0; i < iSquares; i++ )
      {
         ObjectCreate   ( nCellName + i, OBJ_LABEL, nWindow, 0, 0 );
         ObjectSetText  ( nCellName + i, CharToStr ( 110 ), iHeight, "Wingdings", nColor );
         ObjectSet      ( nCellName + i, OBJPROP_XDISTANCE, nX + iXSpace * i );
         ObjectSet      ( nCellName + i, OBJPROP_YDISTANCE, nY );
         ObjectSet      ( nCellName + i, OBJPROP_BACK, true );
      }
   }
   else
   {
      iSquares = MathCeil ( nHeight / nWidth ); // Number of squares used.
      iHeight  = MathRound ( ( nHeight * 100 ) / 77 ); // Real height size.
      iWidth   = MathRound ( ( nWidth * 100 ) / 77 ); // Real width size.
      iXSpace  = iHeight / iSquares - ( ( iWidth / ( 9 - ( nWidth / 100 ) ) ) * 2 );

      for ( i = 0; i < iSquares; i++ )
      {
         ObjectCreate   ( nCellName + i, OBJ_LABEL, nWindow, 0, 0 );
         ObjectSetText  ( nCellName + i, CharToStr ( 110 ), iWidth, "Wingdings", nColor );
         ObjectSet      ( nCellName + i, OBJPROP_XDISTANCE, nX );
         ObjectSet      ( nCellName + i, OBJPROP_YDISTANCE, nY + iXSpace * i );
         ObjectSet      ( nCellName + i, OBJPROP_BACK, true );
      }
   }
}

void DeleteCell(string name)
{
   int square = 0;
   while ( ObjectFind( name + square ) > -1 )
   {
      ObjectDelete( name + square );
      square++;
   }   
}

void DrawBullet(int window, string cellName, int col, int row, color bulletColor )
{
   ObjectCreate   ( cellName, OBJ_LABEL, window, 0, 0 );
   ObjectSetText  ( cellName, CharToStr ( 108 ), 9, "Wingdings", bulletColor );
   ObjectSet      ( cellName, OBJPROP_XDISTANCE, col );
   ObjectSet      ( cellName, OBJPROP_YDISTANCE, row );
   ObjectSet      ( cellName, OBJPROP_BACK, true );
}


